@@ -123,6 +123,7 @@ eim <- function(X = NULL, W = NULL, json_path = NULL) {
         W <- as.matrix(matrices$W)
         allowed_params <- c(
             "prob",
+            "avg_prob",
             "initial_prob",
             "iterations",
             "nboot",
@@ -504,6 +505,16 @@ run_em <- function(object = NULL,
         base_call_sym$W <- object$X
         base_call_sym$json_path <- NULL
         base_call_sym$object <- NULL
+        base_call_sym$scale_factor <- 1
+        ip <- all_params$initial_prob
+        if (is.matrix(ip)) {
+            col_tot_X <- colSums(object$X)
+            num <- sweep(ip, 2, col_tot_X, "*")
+            denom <- rowSums(num)
+            base_call_sym$initial_prob <- sweep(num, 1, denom, "/")
+            base_call_sym$initial_prob <- t(base_call_sym$initial_prob)
+        }
+
 
         inverse <- eval(base_call_sym, parent.frame())
         # --- Reversed features ---
@@ -559,7 +570,7 @@ run_em <- function(object = NULL,
 #' @seealso The [eim] object and [run_em] implementation.
 #'
 #' @return
-#' Returns an `eim` object with the `sd` field containing the estimated standard deviations of the probabilities and the amount of iterations that were made. If an `eim` object is provided, its attributes (see [run_em]) are retained in the returned object.
+#' Returns an `eim` object with the `sd` field containing the estimated standard deviations of the probabilities and the `avg_prob` field with the average bootstrapped probability matrix. If an `eim` object is provided, its attributes (see [run_em]) are retained in the returned object.
 #'
 #' @examples
 #' \donttest{
@@ -737,10 +748,13 @@ bootstrap <- function(object = NULL,
         if (is.matrix(initial_prob)) initial_prob else matrix(-1, nrow = 1, ncol = 1)
     )
 
-    object$sd <- result
+    object$sd <- result$sd
     dimnames(object$sd) <- list(colnames(W), colnames(object$X))
+    object$avg_prob <- result$avg_prob
+    dimnames(object$avg_prob) <- list(colnames(W), colnames(object$X))
     object$nboot <- nboot
     object$sd[object$sd == 9999] <- Inf
+    object$avg_prob[object$avg_prob == 9999] <- Inf
 
     class(object) <- "eim"
     return(object)
@@ -1504,6 +1518,9 @@ summary.eim <- function(object, ...) {
     if (!is.null(object$sd)) {
         object_run_em_attr$sd <- object$sd
     }
+    if (!is.null(object$avg_prob)) {
+        object_run_em_attr$avg_prob <- object$avg_prob
+    }
 
     final_list <- c(object_core_attr, object_run_em_attr)
     final_list
